///////////////////////////////////////////////////////////////////////////////////////
//NAME:     initPLL_SDRAM.asm
//DATE:     6/27/2006
//AUTHOR:   modified by RVC
//PURPOSE:  Set up PLL and SDRAM controller settings for ADSP-21375 QFP BUB
//DETAILS:  Sets up the SDRAM controller to access SDRAM. In this file are two subroutines, the first
//          to set up the SHARC's PLL, and the second to set up the SDRAM controller.
//          CLKIN= 12.5 MHz, Multiplier= 27, Divisor= 2, CCLK_SDCLK_RATIO 2.0.
//          Core clock = (12.5MHz * 21)  = 262.5 MHz
////////////////////////////////////////////////////////////////////////////////////////

#include <def21375.h>

.global _initPLL;
.global _initSDRAM;

.section/pm seg_pmco;

_initPLL:

// CLKIN= 12.5 MHz, Multiplier= 21, Divisor= 1
//    Core clock = (12.5MHz * 21) = 262.5 MHz
    ustat3 = PLLM21;
//ustat3 = PLLM22;

changePLL:

    // Set the Core clock (CCLK) to SDRAM clock (SDCLK) ratio to 2
   bit set ustat3 SDCKR2;

    dm(PMCTL) = ustat3;
    bit set ustat3 PLLBP;

    dm(PMCTL) = ustat3;

     // Wait for at least 4096 cycles for the pll to lock
    lcntr = 5000, do loopend2 until lce;
loopend2:     nop;

    ustat3 = dm(PMCTL);
    bit clr ustat3 PLLBP;
    dm(PMCTL) = ustat3;

_initPLL.end: rts;


_initSDRAM:


    ustat1 = 0x0c; // disable SDCLK0 and 1
    dm(SDCTL) = ustat1;
    
    lcntr = 100, do(PC,1) until lce;
	nop;

	ustat1 = 0x00030616;  // before program SDRRC register, as per SPEC just to be safe
	dm(SDRRC) = ustat1;
	
    lcntr = 100, do(PC,1) until lce;   
	nop;	
  
    ustat3 =  0x2305037B;    // new setting with bank column width = 8 bits and x16 SDRAM
//  ustat3 =  0x2305137B;  // new setting with bank column width = 9 bits and x16 SDRAM
    dm(SDCTL) = ustat3;  
    
    lcntr = 100, do(PC,1) until lce;
	nop;    



    ustat3 = 0xE1;  // map MS0 to SDRAM
    dm(EPCTL) = ustat3;
    
    lcntr = 100, do(PC,1) until lce;
	nop;


// uncomment to observe performance with cache turned off - warning : verrry slow!!
/* 	
	bit set MODE2 CADIS;
    lcntr = 100, do(PC,1) until lce;
	nop;	
*/




_initSDRAM.end: rts;


